from abc import ABC, abstractmethod

from pyRealtor.geo import GeoLocationService


class Realtor(ABC):
    @abstractmethod
    def set_geo_coordinate_boundry(self, geo_location_obj: GeoLocationService):
        pass

    @abstractmethod
    def set_sort_method(self, by: str, ascending_order: bool):
        pass

    @abstractmethod
    def set_transaction_type(self, transaction_type: str):
        pass

    @abstractmethod
    def set_min_amount(self, new_amount, col_name):
        pass

    @abstractmethod
    def set_open_house_only(self, open_house_date):
        pass


    @abstractmethod
    def search_houses(self, use_proxy = False):
        pass





import requests

class GeoLocationService:

    def __init__(self):
        self.nominatim_api_endpoint = "https://nominatim.openstreetmap.org/search"
        self.latitude_max = None
        self.latitude_min = None
        self.longitude_max = None
        self.longitude_min = None
        self.polygon_boundry = None


    def set_geo_location_boundry(self, geo_json):

        geo_coord_1 = (
            float(geo_json['boundingbox'][0]),
            float(geo_json['boundingbox'][2])
        )
        geo_coord_2 = (
            float(geo_json['boundingbox'][1]),
            float(geo_json['boundingbox'][3])
        )

        if not ((geo_coord_1[0] >= -90 and geo_coord_1[0] <= 90) and (geo_coord_2[0] >= -90 and geo_coord_2[0] <= 90)):
            raise ValueError("Latitude of geo co-ordinate must be between -90 and 90")
        
        if not ((geo_coord_1[1] >= -180 and geo_coord_1[1] <= 180) and (geo_coord_2[1] >= -180 and geo_coord_2[1] <= 180)):
            raise ValueError("Longitude of geo co-ordinate must be between -90 and 90")
        
        self.latitude_min = min(geo_coord_1[0], geo_coord_2[0])
        self.latitude_max = max(geo_coord_1[0], geo_coord_2[0])
        self.longitude_min = min(geo_coord_1[1], geo_coord_2[1])
        self.longitude_max = max(geo_coord_1[1], geo_coord_2[1])

        if "geojson" in geo_json:
            self.polygon_boundry = geo_json["geojson"]

    def set_display_physical_location(self, physical_location: str):
        self.physical_location = physical_location

    def get_country(self, city:str):
        country_name = ""
        max_importance = 0
        api_params = {
            'city': city,
            'addressdetails': 1,
            'format': 'json'
        }

        try:
            geo_res = requests.get(
                self.nominatim_api_endpoint,
                params = api_params,
                headers={
                    'User-Agent': 'pyRealtor'
                }
            )
            for geo_json in geo_res.json():
                if geo_json["importance"] >= max_importance:
                    country_name = geo_json["address"]["country"]

        except Exception as e:
            print(e)
            raise

        return country_name


           
    def search_geo_location(self, city: str, country:str, province:str = None):
        geo_res_json = None
        api_params = {
            'city': city,
            'country': country,
            'format': 'json'
        }
        is_polygon_boundry = False

        if province:
            api_params['state'] = province

        if country.lower() == 'united states':
            api_params['polygon_geojson'] = 1
            is_polygon_boundry = True

        try:
            geo_res = requests.get(
                self.nominatim_api_endpoint,
                params = api_params,
                headers={
                    'User-Agent': 'pyRealtor'
                }
            )
            
            geo_res_json_lst = geo_res.json()
            geo_res_json = None
            max_importance = 0

            if len(geo_res_json_lst) == 0:
                raise Exception(f"pyRealtor is only for Canadian or United States region, area {city} not found in either Canada or United States")

            for geo_json in geo_res_json_lst:
                if is_polygon_boundry:
                    if (geo_json["importance"] >= max_importance) and ("geojson" in geo_json) and ("polygon" in geo_json["geojson"]["type"].lower()):
                        geo_res_json = geo_json
                        max_importance = geo_json["importance"]

                elif geo_json["importance"] >= max_importance:
                    geo_res_json = geo_json
                    max_importance = geo_json["importance"]

        except Exception as e:
            print(e)
            raise
        
        return geo_res_json


import requests
from requests.adapters import HTTPAdapter, Retry

from pyRealtor.geo import GeoLocationService
from pyRealtor.report import ReportingService
from pyRealtor.proxy import Proxy
from pyRealtor.realtor import Realtor

class RealtorCa(Realtor):

    def __init__(self, report_obj: ReportingService):
        self.search_api_endpoint = "https://api2.realtor.ca/Listing.svc/PropertySearch_Post"
        self.search_api_params = {
            'Version': '7.0',
            'ApplicationId': '1',
            'CultureId': '1',
            'Currency': 'CAD',
            'RecordsPerPage': 200,
            'MaximumResults': 600
        }
        self.search_api_headers = {
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Accept-Language": "en-US,en;q=0.9",
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
            "DNT": "1",
            "Host": "api2.realtor.ca",
            "Origin": "https://www.realtor.ca",
            "Pragma": "no-cache",
            "Referer": "https://www.realtor.ca/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-site",
            'User-Agent': "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Mobile Safari/537.36"
        }
        self.report_obj = report_obj

    def set_geo_coordinate_boundry(self, geo_location_obj: GeoLocationService):
        try:
            self.search_api_params["LatitudeMin"] = geo_location_obj.latitude_min
            self.search_api_params["LongitudeMin"] = geo_location_obj.longitude_min
            self.search_api_params["LatitudeMax"] = geo_location_obj.latitude_max
            self.search_api_params["LongitudeMax"] = geo_location_obj.longitude_max
            self.search_api_params["ZoomLevel"] = 10
        except Exception as e:
            raise

    def set_sort_method(self, by: str = 'listing_price', ascending_order: bool=True):
        if by.lower() not in ["listing_price", "listing_date_posted"]:
            raise ValueError("Expected value for sort arguments are 'listing_price' or 'listing_date_posted'")

        realtor_sorting_api_str = None

        try:
            if by.lower() == 'listing_price':
                realtor_sorting_api_str = "1"
            elif by.lower() == 'listing_date_posted':
                realtor_sorting_api_str = "6"
            
            if ascending_order:
                realtor_sorting_api_str += "-A"
            else:
                realtor_sorting_api_str += "-D"
            
            self.search_api_params["Sort"] = realtor_sorting_api_str
        except Exception as e:
            raise

    def set_transaction_type(self, transaction_type: str = 'for_sale'):
        if transaction_type.lower() not in ["for_sale", "for_rent", "for_sale_or_rent"]:
            raise ValueError(f"Possible values for Transaction Type argument are 'for_sale' or 'for_rent', however received {transaction_type}")
        
        transaction_type_to_id_dict = {
            'for_sale': '2',
            'for_rent': '3'
        }

        self.search_api_params["TransactionTypeId"] = transaction_type_to_id_dict[transaction_type.lower()]
        
        if transaction_type.lower() == "for_sale":
            keep_col = "Price"
            remove_col = "Rent"
        else:
            keep_col = "Rent"
            remove_col = "Price"

        if remove_col in self.report_obj.column_lst:
            self.report_obj.column_lst.remove(remove_col)
        if keep_col not in self.report_obj.column_lst:
            self.report_obj.column_lst.append(keep_col)

    def set_built_year_range(self, built_year_range: tuple):
        built_year_min, built_year_max = built_year_range
        if built_year_min:
            self.search_api_params["BuildingAgeMin"] = built_year_min
        if built_year_max:
            self.search_api_params["BuildingAgeMax"] = built_year_max

    def set_open_house_only(self, open_house_date):
        self.search_api_params["OpenHouse"] = 1
        self.search_api_params["OpenHouseStartDate"] = open_house_date
        self.search_api_params["OpenHouseEndDate"] = open_house_date

    def set_min_amount(self, new_amount, col_name):
        if col_name not in ['Price', 'Rent']:
            raise Exception(f"col_name must be either Price or Rent, however found {col_name}")
        api_param_key = f"{col_name}Min"
        self.search_api_params[api_param_key] = new_amount
    
    def search_houses(self, use_proxy = False):
        search_result = None
        json_search_payload = self.search_api_params.copy()
        current_page_number = 1


        try:
            s = requests.Session()
            retries = Retry(
                total=3,
                backoff_factor=0.1,
                status_forcelist=[ 500, 502, 503, 504 ]
            )

            s.mount('http://', HTTPAdapter(max_retries=retries))
            
            
            get_api_response = s.get(
                'https://realtor.ca/',
                headers = {
                    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
                    'Accept-Encoding': 'gzip, deflate, br',
                    'Accept-Language': 'en-US,en;q=0.9',
                    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36'
                }
            )

            #print(get_api_response)

            json_search_payload['CurrentPage'] = str(current_page_number)
            s.headers.update(self.search_api_headers)

            def update_reese84_cookie(sess):
                resp = requests.post(
                    "https://www.realtor.ca/dnight-Exit-shall-Braith-Then-why-vponst-is-proc",
                    json = {
                        "solution":{
                                "interrogation":None,
                                "version":"beta"
                            },
                            "old_token":None,
                            "error":None,
                            "performance":{"interrogation":1897}
                    },
                    params = {"d": "www.realtor.ca"}
                )

                if resp.status_code == 200:
                    sess.headers.update({
                        "Cookie": f"reese84={resp.json()['token']};"
                    })
                else:
                    raise Exception(f"Status code {resp.status_code} received while updating reese84 cookie")
            
                return sess
            
            s = update_reese84_cookie(sess=s)

            if use_proxy:
                proxy = Proxy()
                proxy.set_proxies()

                while proxy.rotate_proxy():
                    try:
                        print(f"Using proxy with IP Address: {proxy.current_proxy}")
                        realtor_api_response = s.post(
                            self.search_api_endpoint,
                            headers=self.search_api_headers,
                            data = json_search_payload,
                            proxies = {
                                'http': proxy.current_proxy, 
                                'https': proxy.current_proxy
                            }
                        )

                        if realtor_api_response.status_code == 200:
                            break
                        elif realtor_api_response.status_code == 403:
                            print(f"Proxy {proxy.current_proxy} is blocked by REALTOR.CA")
                            continue

                    except requests.exceptions.ChunkedEncodingError as chunkEncodingException:
                        continue
                    except requests.exceptions.ProxyError as proxyException:
                        continue
                    except requests.exceptions.ConnectionError as connectionException:
                        continue
                
            else:
                realtor_api_response = s.post(
                    self.search_api_endpoint,
                    headers=self.search_api_headers,
                    data = json_search_payload
                )

            
            if realtor_api_response.status_code == 200:
                search_result = realtor_api_response.json()
                self.report_obj.house_json_lst.extend(search_result["Results"])
                total_available_pages = search_result["Paging"]["TotalPages"]
     
                while current_page_number < total_available_pages:
                    current_page_number += 1
                    json_search_payload['CurrentPage'] = str(current_page_number)
                    
                    s = update_reese84_cookie(sess=s)
                    #time.sleep(1000)

                    if use_proxy:
                        realtor_api_response = None
                        while proxy.rotate_proxy():
                            try:
                                print(f"Using proxy with IP Address: {proxy.current_proxy}")
                                realtor_api_response = s.post(
                                    self.search_api_endpoint,
                                    headers=self.search_api_headers,
                                    data = json_search_payload,
                                    proxies = {
                                        'http': proxy.current_proxy, 
                                        'https': proxy.current_proxy
                                    }
                                )
                                if realtor_api_response.status_code == 200:
                                    break
                                elif realtor_api_response.status_code == 403:
                                    print(f"Proxy {proxy.current_proxy} is blocked by REALTOR.CA")
                                    continue
                                else:
                                    continue
                                
                            except requests.exceptions.ChunkedEncodingError as chunkEncodingException:
                                continue

                            except requests.exceptions.ProxyError as proxyException:
                                continue

                            except requests.exceptions.ConnectionError as connectionException:
                                continue

                    else:
                        realtor_api_response = s.post(
                            self.search_api_endpoint,
                            headers=self.search_api_headers,
                            data = json_search_payload
                        )
                        #print(s.cookies.get_dict())

                    if realtor_api_response is None:
                        listings_received = len(self.report_obj.house_json_lst)
                        print(f"Total listings received: {listings_received}, no more proxies available to connect, please try after some time")
                    elif realtor_api_response.status_code == 200:
                        search_result = realtor_api_response.json()
                        self.report_obj.house_json_lst.extend(search_result["Results"])
                    
            elif realtor_api_response.status_code == 403:
                print(realtor_api_response.text)
                print("IP address is blocked by REALTOR.CA, please try using Proxy with parameter use_proxy=True")

        except Exception as e:
            raise

        return self.report_obj


import json
import importlib.resources
import pkg_resources
import re
import sys

import pandas as pd

class ReportingService:  

    def __init__(
            self, 
            column_mapping_cfg_fpath:str = 'realtor_json_normalize_mapping.json', 
            column_lst: list = [
                'MLS', 'Description', 'Bathrooms', 'Bedrooms', 'Size', 'Stories', 
                'House Category', 'Ammenities', 
                'Price', 'Address', 'Latitude', 'Longitude', 'Ownership Category', 'Nearby Ammenities', 'Open House', 'Website'
            ],
            summary_col_lst: list = ['Bedrooms', 'Bathrooms', 'House Category', 'Ownership Category']
    ):
        self.house_json_lst = []
        self.column_lst = column_lst
        self.summary_col_lst = summary_col_lst
        self.column_mapping_dict = self.load_mapping_cfg(
            config_path=column_mapping_cfg_fpath
        )

    def load_mapping_cfg(self, config_path:str):
        config_dict = {}
        
        with pkg_resources.resource_stream('pyRealtor', config_path) as f:
            config_dict = json.load(f)
        
        return config_dict


    def to_dataframe(self):
        dataframe_values = []
        
        for mls_listing_json in self.house_json_lst:
            dataframe_row_values = []
            for col_to_report in self.column_lst:
                json_copy = mls_listing_json.copy()

                if "->" in self.column_mapping_dict[col_to_report]:
                    col_hierarchy_lst = self.column_mapping_dict[col_to_report].split("->")
                    if (len(col_hierarchy_lst) == 2) and (col_hierarchy_lst[0] in json_copy):
                        json_copy = ";".join(
                            filter(
                                None,
                                [
                                    ele[col_hierarchy_lst[1]] if col_hierarchy_lst[1] in ele else 
                                    ele[col_hierarchy_lst[1].split(".")[0]][col_hierarchy_lst[1].split(".")[1]] if (col_hierarchy_lst[1].split(".")[0] in ele) and (col_hierarchy_lst[1].split(".")[1] in ele[col_hierarchy_lst[1].split(".")[0]]) else 
                                    ''
                                    for ele in json_copy[col_hierarchy_lst[0]] 
                                ]
                            )
                        )
                    else:
                        json_copy = ''
                    
                else:
                    col_hierarchy_lst = self.column_mapping_dict[col_to_report].split(".")
                    for col_hierarchy in col_hierarchy_lst:
                        if col_hierarchy in json_copy:
                            json_copy = json_copy[col_hierarchy]
                        else:
                            json_copy = ''
                dataframe_row_values.append(json_copy)
            dataframe_values.append(dataframe_row_values)
        
        listing_dataframe = pd.DataFrame(dataframe_values, columns=self.column_lst)
        self.house_json_lst = []

        return listing_dataframe

    def get_average(self, dataframe: pd.DataFrame, average_col_name: str):
        dataframe[average_col_name] = pd.to_numeric(dataframe[average_col_name])
        dataframe['Bedrooms'] = dataframe['Bedrooms'].fillna('').astype("string")

        dataframe['Bedrooms'] = dataframe['Bedrooms'].apply(
            lambda row: 0 if row=='' else eval(row)
        )
        summary_df = dataframe.groupby(
            self.summary_col_lst,
            as_index = True
        )[average_col_name].mean()

        return summary_df

    def save_excel(self, dataframe_to_report, file_path, summary_dataframe):
        
        with pd.ExcelWriter(file_path) as excel_writer:
            summary_dataframe.to_excel(excel_writer, sheet_name = 'Summary', freeze_panes = (1,0))
            dataframe_to_report.to_excel(excel_writer, sheet_name = 'Listings', index=False, freeze_panes = (1,0))



def test(listing_dataframe):
    print(listing_dataframe['Bedrooms'].tail(20))
    listing_dataframe['Bedrooms'] = listing_dataframe['Bedrooms'].apply(lambda row: 0 if row=='' else eval(row))
    print(listing_dataframe['Bedrooms'].tail(20))
    summary_df = listing_dataframe.groupby(
                ['Bedrooms', 'Bathrooms', 'House Category', 'Ownership Category'],
                as_index = True
            )['Price'].mean()
    print(summary_df)

    #print(summary_df.get_group(('3', '1', 'Row / Townhouse', 'Condominium/Strata'))[['mls', 'listing_price']])
    #print(summary_df.get_group(('3', '1', 'Row / Townhouse', 'Condominium/Strata'))['listing_price'].mean())

    #summary_df.to_excel('../summary_test_1.xlsx')

if __name__ == "__main__":
    current_df = pd.read_excel('../ottawa_all_listings.xlsx', sheet_name='Listings', na_filter = False)
    test(listing_dataframe=current_df.copy())


import os
import pandas as pd
import datetime

from pyRealtor.geo import GeoLocationService
from pyRealtor.realtorFactory import RealtorFactory
from pyRealtor.report import ReportingService

class HousesFacade:

    def search_save_houses(
        self,
        search_area: str,
        report_file_name: str = None,
        country: str = None,
        listing_type: str = 'for_sale',
        use_proxy: bool = False, 
        get_summary: bool = True,
        price_from: int = None,
        sorted_col_name:str = None,
        realtor_name_filter: str = None,
        realtor_brokerage_filter: str = None,
        column_mapping_cfg_fpath:str = None, 
        column_lst: list = None,
        **kwargs
    ):
        
        geo_service_obj = GeoLocationService()

        if country is None:
            country = geo_service_obj.get_country(city = search_area)

        realtor_service_obj = RealtorFactory().get_realtor(
            country = country,
            config = column_mapping_cfg_fpath
        )

        geo_result_json = geo_service_obj.search_geo_location(city=search_area, country=country)

        display_address = geo_result_json["name"]
        geo_service_obj.set_display_physical_location(display_address)
        realtor_service_obj.set_transaction_type(listing_type)
    
        geo_service_obj.set_geo_location_boundry(geo_result_json)

        if report_file_name is None:
            report_file_name = f"{geo_service_obj.physical_location}_{listing_type.lower()}_{datetime.datetime.now().strftime('%Y-%m-%d_%H_%M')}.xlsx"

        current_directory = os.getcwd()
        file_path_to_save = os.path.join(current_directory, report_file_name)

        try:
            realtor_service_obj.set_geo_coordinate_boundry(geo_service_obj)
        except Exception:
            raise ValueError(f"Area: {search_area} in Country: {country} does not fit geographic requirements") 
        
        if price_from is not None:
            realtor_service_obj.set_min_amount(
                new_amount = int(price_from),
                col_name = sorted_col_name
            )

        if sorted_col_name is not None:
            realtor_service_obj.set_sort_method(by=sorted_col_name, ascending_order=True)

        if 'open_house_date' in kwargs:
            open_house_date = kwargs['open_house_date']
            realtor_service_obj.set_open_house_only(open_house_date)

        if (realtor_name_filter or realtor_brokerage_filter):
            if 'Realtor Name' not in column_lst:
                realtor_service_obj.report_obj.column_lst.append('Realtor Name')
            if 'Realtor Brokerage' not in column_lst:
                realtor_service_obj.report_obj.column_lst.column_lst.append('Realtor Brokerage')

        houses_df = realtor_service_obj.search_houses(
            use_proxy
        ).to_dataframe()

        loop_counter = 0

        if houses_df.shape[0] > 0 and sorted_col_name == 'listing_price':

            current_min_amount = pd.to_numeric(
                houses_df[sorted_col_name],
                downcast="integer"
            ).values[-1]

            while True:
                print(f"Fetching listings with minimum price: {current_min_amount}")
                
                if loop_counter > 10:
                    print(f"The Maximun Limit has reached. Please use the function with parameter price_from={current_min_amount} after some time")
                    break
                
                realtor_service_obj.set_min_amount(
                    new_amount = int(current_min_amount),
                    col_name = sorted_col_name
                )

                try:
                    new_houses_df = realtor_service_obj.search_houses(
                        use_proxy
                    ).to_dataframe()
                except Exception as e:
                    print(f"Exception raised while searching from Minimum Price: {current_min_amount}: {e}")
                    break

                if new_houses_df.shape[0] > 0:
                    new_min_amount = pd.to_numeric(
                        new_houses_df[sorted_col_name],
                        downcast="integer"
                    ).values[-1]

                    if new_min_amount > current_min_amount:
                        houses_df = pd.concat([houses_df, new_houses_df], axis=0).drop_duplicates(subset=['MLS'])
                        current_min_amount = new_min_amount
                    else:
                        break
                else:
                    break

                loop_counter += 1

        if realtor_name_filter and "Realtor Name" in houses_df.columns:
            print(f"Filtering for Realtor Name: {realtor_name_filter}")
            houses_df = houses_df[
                houses_df["Realtor Name"].str.contains(realtor_name_filter, case=False)
            ]
            print(f"Number of listings found after filtering: {houses_df.shape[0]}")
        elif realtor_brokerage_filter and "Realtor Brokerage" in houses_df.columns:
            print(f"Filtering for Realtor's Brokerage Name: {realtor_brokerage_filter}")
            houses_df = houses_df[
                houses_df["Realtor Brokerage"].str.contains(realtor_brokerage_filter, case=False)
            ]
            print(f"Number of listings found after filtering: {houses_df.shape[0]}")

        if get_summary:
            if listing_type == 'for_sale':
                summary_df = realtor_service_obj.report_obj.get_average(
                    dataframe = houses_df.copy(),
                    average_col_name = 'Price',
                )
            elif listing_type == 'for_rent':
                """
                summary_df = realtor_service_obj.report_obj.get_average(
                    dataframe = houses_df,
                    average_col_name = 'Rent',
                    grpby_col_lst = ['Bedrooms', 'Bathrooms', 'House Category', 'Ownership Category']
                )
                """
                summary_df = pd.DataFrame()

        #print(houses_df)

        if houses_df.shape[0] > 0:
            realtor_service_obj.report_obj.save_excel(
                houses_df,
                file_path_to_save,
                summary_df
            )


from pyRealtor.realtorCa import RealtorCa
from pyRealtor.realtorCom import RealtorCom
from pyRealtor.report import ReportingService


class RealtorFactory:

    def get_realtor(self, country: str, config: str = None):

        if config is None and country == "Canada":
            config = "config/column_mapping_cfg.json"
        elif config is None and country == "United States":
            config = "config/column_mapping_cfg_realtor_com.json"

        if country == 'Canada':
            realtor_service_obj = RealtorCa(
                ReportingService(
                    column_mapping_cfg_fpath = config,
                    column_lst = [
                        'MLS', 'Description', 'Bedrooms', 'Bathrooms', 'Size', 'Stories', 
                        'House Category', 'Ammenities', 
                        'Price', 'Address', 'Latitude', 'Longitude', 'Ownership Category', 'Nearby Ammenities', 'Open House', 'Website'
                    ],
                    summary_col_lst = ['Bedrooms', 'Bathrooms', 'House Category', 'Ownership Category']
                )
            )
        elif country == 'United States':
            realtor_service_obj = RealtorCom(
                ReportingService(
                    column_mapping_cfg_fpath = config,
                    column_lst = [
                        'ID', 'Bathrooms', 'Bedrooms', 'Size', 'House Category',
                        'Price', 'street name', 'city', 'state', 'Latitude', 'Longitude', 'InsertedDate'
                    ],
                    summary_col_lst = ['Bedrooms', 'Bathrooms', 'House Category']
                )
            )
        else:
            raise ValueError(f"Expected countries are United States or Canada, however received {country}")
        
        return realtor_service_obj


import requests
import logging
import time

class Proxy:
    def __init__(self, username=None, password=None, host=None, port=None):
        # Configure logging
        self.logger = self._setup_logger()
        
        # Proxy configuration
        self.PROXY_USERNAME = username
        self.PROXY_PASSWORD = password
        self.PROXY_HOST = host
        self.PROXY_PORT = port
        
        self.current_proxy = None
        self.max_retries = 3
        self.retry_delay = 2  # seconds between retries

    def _setup_logger(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s [%(levelname)s] %(message)s',
            handlers=[
                logging.FileHandler('proxy.log'),
                logging.StreamHandler()
            ]
        )
        return logging.getLogger(__name__)

    def get_proxy_url(self):
        if self.PROXY_USERNAME and self.PROXY_PASSWORD and self.PROXY_HOST and self.PROXY_PORT:
            return f"http://{self.PROXY_USERNAME}:{self.PROXY_PASSWORD}@{self.PROXY_HOST}:{self.PROXY_PORT}"
        else:
            return None

    def set_proxies(self):
        # No need to fetch free proxies anymore
        self.current_proxy = self.get_proxy_url()
        self.logger.info("Custom proxy configuration loaded")
        return True

    def rotate_proxy(self):
        # Since we're using a single proxy, we'll just verify it works
        proxy_str = self.get_proxy_url()
        
        for attempt in range(self.max_retries):
            try:
                proxy_check_resp = requests.get(
                    "https://httpbin.org/ip", 
                    proxies={
                        'http': proxy_str,
                        'https': proxy_str
                    },
                    timeout=10
                )

                if proxy_check_resp.status_code == 200:
                    self.logger.info(f"Proxy test successful: {proxy_check_resp.json()}")
                    self.current_proxy = proxy_str
                    return True
                    
            except Exception as e:
                self.logger.error(f"Proxy test attempt {attempt + 1} failed: {str(e)}")
                if attempt < self.max_retries - 1:
                    time.sleep(self.retry_delay)
                continue

        self.logger.error("All proxy test attempts failed")
        return False

if __name__ == "__main__":
    # Example usage
    proxy = Proxy()  # No credentials provided
    proxy.set_proxies()
    if proxy.rotate_proxy():
        print(f"Successfully connected using proxy: {proxy.current_proxy}")
    else:
        print("Failed to establish proxy connection")



from Realtorpy.geo import GeoLocationService
from Realtorpy.realtor import Realtor
from Realtorpy.report import ReportingService
from Realtorpy.facade import HousesFacade
from Realtorpy.proxy import Proxy
from Realtorpy.realtorCa import RealtorCa
from Realtorpy.realtorCom import RealtorCom
from Realtorpy.realtorFactory import RealtorFactory

__version__ = "1.0.0"

__all__ = [
    "GeoLocationService",
    "Realtor",
    "ReportingService",
    "HousesFacade",
    "Proxy",
    "RealtorCa",
    "RealtorCom",
    "RealtorFactory"
]


# Copyright (C) 2024 Lovepreet Gill <labi1240@gmail.com>
# License: MIT, labi1240@gmail.com

from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name='Realtorpy',
    version='1.0.0',
    description='Python package for fetching and analyzing REALTOR.CA and REALTOR.COM MLS Listings',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Lovepreet Gill',
    author_email='labi1240@gmail.com',
    license='MIT',
    url='https://github.com/labi1240/Realtorpy',
    packages=find_packages(),
    python_requires='>=3.6',
    install_requires=[
        'requests>=2.26.0',
        'pandas>=1.3.5',
        'numpy>=1.21.4',
        'openpyxl>=3.1.2',
        'lxml>=5.1.0',
        'pyyaml>=6.0.1'
    ],
    extras_require={
        "dev": ["twine"]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    include_package_data=True,
    package_data={
        'Realtorpy': [
            'config/column_mapping_cfg.json',
            'config/column_mapping_cfg_realtor_com.json',
            'config/graphql_queries.yml'
        ]
    }
)


import requests
from requests.adapters import HTTPAdapter, Retry
import pkg_resources
import yaml
import json
import datetime

from pyRealtor.geo import GeoLocationService
from pyRealtor.report import ReportingService
from pyRealtor.proxy import Proxy
from pyRealtor.realtor import Realtor

class RealtorCom(Realtor):

    def __init__(self, report_obj: ReportingService):
        self.search_api_endpoint = "https://www.realtor.com/api/v1/rdc_search_srp"
        self.search_api_params = {
            'client_id': 'rdc-search-for-sale-search',
            'schema': 'vesta'
        }
        self.search_api_headers = {
            "Accept": "application/json, text/javascript",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Accept-Language": "en-US,en;q=0.9",
            "Content-Type": "application/json",
            "Origin": "https://www.realtor.com",
            "Sec-Fetch-Site": "same-origin"
        }
        self.search_api_body = {
            "query": "",
            "variables": {
                "limit": 200,
                "offset": 0,
                "client_data": {
                    "device_data": {
                        "device_type": "desktop"
                    }
                },
                "query": {
                    "primary": True,
                    "status": [],
                    "boundary": {}
                }
            }
        }
        self.report_obj = report_obj

    def set_geo_coordinate_boundry(self, geo_location_obj: GeoLocationService):
        if geo_location_obj.polygon_boundry:
            self.search_api_body["variables"]["query"]["boundary"] = geo_location_obj.polygon_boundry
        else:
            raise TypeError("REALTOR.COM expects Polygon boundary to search real estate listings")

    def set_sort_method(self, by: str = 'relevant', ascending_order: bool=True):
        if by.lower() not in ["listing_price", "listing_date_posted", "relevant"]:
            raise ValueError("Expected value for sort arguments are 'relevant', 'listing_price' or 'listing_date_posted'")
        
        order = ''
        if ascending_order:
            order = "asc"
        else:
            order = "desc"


        if by.lower() == "relevant":
            self.search_api_body["variables"]["sort_type"] = "relevant"
        elif by.lower() == "listing_price":
            self.search_api_body["variables"]["sort"] = [{
                "field": "list_price",
                "direction": order
            }]
        elif by.lower() == "listing_date_posted":
            self.search_api_body["variables"]["sort"] = [{
                "field": "list_date",
                "direction": order
            }]


    def set_transaction_type(self, transaction_type: str = 'for_sale'):
        if transaction_type.lower() not in ["for_sale", "for_rent"]:
            raise ValueError(f"Possible values for Transaction Type argument are 'for_sale' or 'for_rent', however received {transaction_type}")

        if transaction_type.lower() == 'for_sale':
            self.search_api_params["client_id"] = "rdc-search-for-sale-search"
            self.search_api_body["variables"]["query"]["status"].append("for_sale")
        else:
            self.search_api_params["client_id"] = "rdc-search-rentals-search"
            self.search_api_body["variables"]["query"]["status"].append("for_rent")

    def set_min_amount(self, new_amount: float, col_name: str = None):
        self.search_api_body["variables"]["query"]["list_price"] = {
            "min": new_amount
        }

    def set_open_house_only(self, open_house_date: str):
        if open_house_date != datetime.datetime.strptime(open_house_date, "%Y-%m-%d").strftime("%Y-%m-%d"):
            raise ValueError(f"Open House Date is Expected in YYYY-MM-DD format, however received {open_house_date}")
        
        self.search_api_params["open_house"]["min"] = open_house_date
        self.search_api_params["open_house"]["max"] = open_house_date


    def search_houses(self, use_proxy = False):
        search_result = None
        self.search_api_body["query"] = ""

        with pkg_resources.resource_stream('pyRealtor', 'config/graphql_queries.yml') as graph_ql_yaml:
            try:
                search_params_dict = yaml.safe_load(graph_ql_yaml)
                search_query = search_params_dict["search_function_graphql"]
                search_cols = search_params_dict["search_houses_columns"]

                search_cols.format(
                        EXTRA_PROPERTIES_COLUMNS = ''
                    )
                
                search_query_formatted = search_query.format(
                    SEARCH_COLUMNS = search_cols.format(
                        EXTRA_PROPERTIES_COLUMNS = ''
                    )
                )
            except yaml.YAMLError as exc:
                print(exc)
                raise

        self.search_api_body["query"] = search_query_formatted

        json_search_payload = self.search_api_body.copy()

        current_limit = 200
        current_offset = 0

        try:
            s = requests.Session()
            retries = Retry(
                total=3,
                backoff_factor=0.1,
                status_forcelist=[ 500, 502, 503, 504 ]
            )

            s.mount('https://', HTTPAdapter(max_retries=retries))
            s.headers.update(self.search_api_headers)

            json_search_payload["variables"]["limit"] = current_limit
            json_search_payload["variables"]["offset"] = current_offset

            if use_proxy:
                proxy = Proxy()
                proxy.set_proxies()

                while proxy.rotate_proxy():
                    try:
                        print(f"Using proxy with IP Address: {proxy.current_proxy}")
                        realtor_api_response = s.post(
                            self.search_api_endpoint,
                            headers=self.search_api_headers,
                            json = json_search_payload,
                            params = self.search_api_params,
                            proxies = {
                                'http': proxy.current_proxy, 
                                'https': proxy.current_proxy
                            }
                        )

                        if realtor_api_response.status_code == 200:
                            break
                        elif realtor_api_response.status_code == 403:
                            print(f"Proxy {proxy.current_proxy} is blocked by REALTOR.CA")
                            continue

                    except requests.exceptions.ChunkedEncodingError as chunkEncodingException:
                        continue
                    except requests.exceptions.ProxyError as proxyException:
                        continue
                    except requests.exceptions.ConnectionError as connectionException:
                        continue
            else:
                realtor_api_response = s.post(
                    self.search_api_endpoint,
                    headers=self.search_api_headers,
                    json = json_search_payload,
                    params = self.search_api_params
                )

            print(realtor_api_response.status_code)

            if realtor_api_response.status_code == 200:
                search_result = realtor_api_response.json()
                self.report_obj.house_json_lst.extend(search_result["data"]["home_search"]["properties"])

                total_available_listings = search_result["data"]["home_search"]["total"]
                total_current_page_listings = search_result["data"]["home_search"]["count"]

                while total_current_page_listings > 0 or current_offset < total_available_listings:
                    print(current_offset, total_available_listings, total_current_page_listings)

                    current_offset += total_current_page_listings
                    json_search_payload["variables"]['offset'] = current_offset

                    if use_proxy:
                        realtor_api_response = None
                        while proxy.rotate_proxy():
                            try:
                                print(f"Using proxy with IP Address: {proxy.current_proxy}")
                                realtor_api_response = s.post(
                                    self.search_api_endpoint,
                                    headers=self.search_api_headers,
                                    json = json_search_payload,
                                    params = self.search_api_params,
                                    proxies = {
                                        'http': proxy.current_proxy, 
                                        'https': proxy.current_proxy
                                    }
                                )

                                if realtor_api_response.status_code == 200:
                                    break
                                elif realtor_api_response.status_code == 403:
                                    print(f"Proxy {proxy.current_proxy} is blocked by REALTOR.CA")
                                    continue

                            except requests.exceptions.ChunkedEncodingError as chunkEncodingException:
                                continue
                            except requests.exceptions.ProxyError as proxyException:
                                continue
                            except requests.exceptions.ConnectionError as connectionException:
                                continue
                    else:
                        realtor_api_response = s.post(
                            self.search_api_endpoint,
                            headers=self.search_api_headers,
                            json = json_search_payload,
                            params = self.search_api_params
                        )

                    #print(realtor_api_response.status_code)
                    if realtor_api_response is None:
                        listings_received = len(self.report_obj.house_json_lst)
                        print(f"Total listings received: {listings_received}, no more proxies available to connect, please try after some time")
                    elif realtor_api_response.status_code == 200:
                        search_result = realtor_api_response.json()
                        self.report_obj.house_json_lst.extend(search_result["data"]["home_search"]["properties"])

                        total_available_listings = search_result["data"]["home_search"]["total"]
                        total_current_page_listings = search_result["data"]["home_search"]["count"]
                    elif realtor_api_response.status_code == 403:
                        print("IP address is blocked by REALTOR.COM, please try using Proxy with parameter use_proxy=True")

            elif realtor_api_response.status_code == 403:
                print("IP address is blocked by REALTOR.COM, please try using Proxy with parameter use_proxy=True")

        except Exception as e:
            raise

        return self.report_obj